@@ -195,14 +195,14 @@ void inject_into_answer(char *burst, size_t count)
 /* Get a verbatim keystroke and insert it into the answer. */
 void do_statusbar_verbatim_input(void)
 {
+	size_t count = 1;
 	char *bytes;
-	size_t count;
 
 	bytes = get_verbatim_kbinput(bottomwin, &count);
 
-	if (count > 0)
+	if (0 < count && count < 999)
 		inject_into_answer(bytes, count);
-	else
+	else if (count == 0)
 		beep();
 
 	free(bytes);
