@@ -2087,7 +2087,7 @@ bool replace_buffer(const char *filename, undo_type action, const char *operatio
 }
 
 /* Execute the given program, with the given temp file as last argument. */
-const char *treat(char *tempfile_name, char *theprogram, bool spelling)
+void treat(char *tempfile_name, char *theprogram, bool spelling)
 {
 	ssize_t lineno_save = openfile->current->lineno;
 	size_t current_x_save = openfile->current_x;
@@ -2098,14 +2098,21 @@ const char *treat(char *tempfile_name, char *theprogram, bool spelling)
 	long timestamp_nsec = 0;
 	static char **arguments = NULL;
 	pid_t thepid;
-	int program_status;
+	int program_status, errornumber;
 	bool replaced = FALSE;
 
 	/* Stat the temporary file.  If that succeeds and its size is zero,
 	 * there is nothing to do; otherwise, store its time of modification. */
 	if (stat(tempfile_name, &fileinfo) == 0) {
-		if (fileinfo.st_size == 0)
-			return NULL;
+		if (fileinfo.st_size == 0) {
+#ifndef NANO_TINY
+			if (spelling && openfile->mark)
+				statusline(ALERT, _("Selection is empty"));
+			else
+#endif
+				statusbar(_("Buffer is empty"));
+			return;
+		}
 
 		timestamp_sec = (long)fileinfo.st_mtim.tv_sec;
 		timestamp_nsec = (long)fileinfo.st_mtim.tv_nsec;
@@ -2122,22 +2129,26 @@ const char *treat(char *tempfile_name, char *theprogram, bool spelling)
 
 		/* Terminate the child if the program is not found. */
 		exit(9);
-	} else if (thepid < 0)
-		return _("Could not fork");
+	} else if (thepid > 0) {
+		/* Block SIGWINCHes while waiting for the forked program to end,
+		 * so nano doesn't get pushed past the wait(). */
+		block_sigwinch(TRUE);
+		wait(&program_status);
+		block_sigwinch(FALSE);
+	}
 
-	/* Block SIGWINCHes while waiting for the program to end,
-	 * so nano doesn't get pushed past the wait(). */
-	block_sigwinch(TRUE);
-	wait(&program_status);
-	block_sigwinch(FALSE);
+	errornumber = errno;
 
 	/* Restore the terminal state and reenter curses mode. */
 	terminal_init();
 	doupdate();
 
-	if (!WIFEXITED(program_status) || WEXITSTATUS(program_status) > 2) {
+	if (thepid < 0) {
+		statusline(ALERT, _("Could not fork: %s"), strerror(errornumber));
+		return;
+	} else if (!WIFEXITED(program_status) || WEXITSTATUS(program_status) > 2) {
 		statusline(ALERT, _("Error invoking '%s'"), arguments[0]);
-		return NULL;
+		return;
 	} else if (WEXITSTATUS(program_status) != 0)
 		statusline(ALERT, _("Program '%s' complained"), arguments[0]);
 
@@ -2146,7 +2157,7 @@ const char *treat(char *tempfile_name, char *theprogram, bool spelling)
 					(long)fileinfo.st_mtim.tv_sec == timestamp_sec &&
 					(long)fileinfo.st_mtim.tv_nsec == timestamp_nsec) {
 		statusbar(_("Nothing changed"));
-		return NULL;
+		return;
 	}
 
 #ifndef NANO_TINY
@@ -2196,8 +2207,6 @@ const char *treat(char *tempfile_name, char *theprogram, bool spelling)
 		statusbar(_("Finished checking spelling"));
 	else
 		statusbar(_("Buffer has been processed"));
-
-	return NULL;
 }
 #endif /* ENABLE_SPELLER || ENABLE_COLOR */
 
@@ -2321,8 +2330,8 @@ bool fix_spello(const char *word)
 /* Run a spell-check on the given file, using 'spell' to produce a list of all
  * misspelled words, then feeding those through 'sort' and 'uniq' to obtain an
  * alphabetical list, which words are then offered one by one to the user for
- * correction.  Return NULL when okay, and the error string otherwise. */
-const char *do_int_speller(const char *tempfile_name)
+ * correction. */
+void do_int_speller(const char *tempfile_name)
 {
 	char *misspellings, *pointer, *oneword;
 	long pipesize;
@@ -2332,8 +2341,12 @@ const char *do_int_speller(const char *tempfile_name)
 	int spell_status, sort_status, uniq_status;
 
 	/* Create all three pipes up front. */
-	if (pipe(spell_fd) == -1 || pipe(sort_fd) == -1 || pipe(uniq_fd) == -1)
-		return _("Could not create pipe");
+	if (pipe(spell_fd) == -1 || pipe(sort_fd) == -1 || pipe(uniq_fd) == -1) {
+		statusline(ALERT, _("Could not create pipe: %s"), strerror(errno));
+		return;
+	}
+
+	statusbar(_("Invoking spell checker..."));
 
 	/* Fork a process to run spell in. */
 	if ((pid_spell = fork()) == 0) {
@@ -2409,16 +2422,18 @@ const char *do_int_speller(const char *tempfile_name)
 
 	/* When some child process was not forked successfully... */
 	if (pid_spell < 0 || pid_sort < 0 || pid_uniq < 0) {
+		statusline(ALERT, _("Could not fork: %s"), strerror(errno));
 		close(uniq_fd[0]);
-		return _("Could not fork");
+		return;
 	}
 
 	/* Get the system pipe buffer size. */
 	pipesize = fpathconf(uniq_fd[0], _PC_PIPE_BUF);
 
 	if (pipesize < 1) {
+		statusline(ALERT, _("Could not get size of pipe buffer"));
 		close(uniq_fd[0]);
-		return _("Could not get size of pipe buffer");
+		return;
 	}
 
 	/* Leave curses mode so that error messages go to the original screen. */
@@ -2484,17 +2499,13 @@ const char *do_int_speller(const char *tempfile_name)
 	waitpid(pid_uniq, &uniq_status, 0);
 
 	if (WIFEXITED(uniq_status) == 0 || WEXITSTATUS(uniq_status))
-		return _("Error invoking \"uniq\"");
-
-	if (WIFEXITED(sort_status) == 0 || WEXITSTATUS(sort_status))
-		return _("Error invoking \"sort\"");
-
-	if (WIFEXITED(spell_status) == 0 || WEXITSTATUS(spell_status))
-		return _("Error invoking \"spell\"");
-
-	/* When all went okay. */
-	statusbar(_("Finished checking spelling"));
-	return NULL;
+		statusline(ALERT, _("Error invoking \"uniq\""));
+	else if (WIFEXITED(sort_status) == 0 || WEXITSTATUS(sort_status))
+		statusline(ALERT, _("Error invoking \"sort\""));
+	else if (WIFEXITED(spell_status) == 0 || WEXITSTATUS(spell_status))
+		statusline(ALERT, _("Error invoking \"spell\""));
+	else
+		statusbar(_("Finished checking spelling"));
 }
 
 /* Spell check the current file.  If an alternate spell checker is
@@ -2504,7 +2515,6 @@ void do_spell(void)
 	FILE *stream;
 	char *temp_name;
 	unsigned stash[sizeof(flags) / sizeof(flags[0])];
-	const char *result_msg;
 	bool okay;
 
 	ran_a_tool = TRUE;
@@ -2541,9 +2551,9 @@ void do_spell(void)
 	blank_bottombars();
 
 	if (alt_speller)
-		result_msg = treat(temp_name, alt_speller, TRUE);
+		treat(temp_name, alt_speller, TRUE);
 	else
-		result_msg = do_int_speller(temp_name);
+		do_int_speller(temp_name);
 
 	unlink(temp_name);
 	free(temp_name);
@@ -2554,14 +2564,6 @@ void do_spell(void)
 	/* Ensure the help lines will be redrawn and a selection is retained. */
 	currmenu = MMOST;
 	shift_held = TRUE;
-
-	if (result_msg != NULL) {
-		/* Avoid giving a failure reason of "Success". */
-		if (errno == 0)
-			statusline(ALERT, result_msg);
-		else
-			statusline(ALERT, _("%s: %s"), result_msg, strerror(errno));
-	}
 }
 #endif /* ENABLE_SPELLER */
 
@@ -2607,13 +2609,13 @@ void do_linter(void)
 
 	/* Create a pipe up front. */
 	if (pipe(lint_fd) == -1) {
-		statusline(ALERT, _("Could not create pipe"));
+		statusline(ALERT, _("Could not create pipe: %s"), strerror(errno));
 		return;
 	}
 
 	blank_bottombars();
 	currmenu = MLINTER;
-	statusbar(_("Invoking linter, please wait"));
+	statusbar(_("Invoking linter..."));
 
 	construct_argument_list(&lintargs, openfile->syntax->linter, openfile->filename);
 
@@ -2640,8 +2642,8 @@ void do_linter(void)
 
 	/* If the child process was not forked successfully... */
 	if (pid_lint < 0) {
+		statusline(ALERT, _("Could not fork: %s"), strerror(errno));
 		close(lint_fd[0]);
-		statusline(ALERT, _("Could not fork"));
 		return;
 	}
 
@@ -2649,8 +2651,8 @@ void do_linter(void)
 	pipesize = fpathconf(lint_fd[0], _PC_PIPE_BUF);
 
 	if (pipesize < 1) {
-		close(lint_fd[0]);
 		statusline(ALERT, _("Could not get size of pipe buffer"));
+		close(lint_fd[0]);
 		return;
 	}
 
@@ -2914,7 +2916,6 @@ void do_formatter(void)
 	FILE *stream;
 	char *temp_name;
 	bool okay = FALSE;
-	const char *result_msg;
 
 	ran_a_tool = TRUE;
 
@@ -2926,6 +2927,10 @@ void do_formatter(void)
 		return;
 	}
 
+#ifndef NANO_TINY
+	openfile->mark = NULL;
+#endif
+
 	temp_name = safe_tempfile(&stream);
 
 	if (temp_name != NULL)
@@ -2937,10 +2942,7 @@ void do_formatter(void)
 		return;
 	}
 
-	result_msg = treat(temp_name, openfile->syntax->formatter, FALSE);
-
-	if (result_msg != NULL)
-		statusline(ALERT, result_msg);
+	treat(temp_name, openfile->syntax->formatter, FALSE);
 
 	unlink(temp_name);
 	free(temp_name);
@@ -3006,8 +3008,8 @@ void do_wordlinechar_count(void)
 /* Get verbatim input. */
 void do_verbatim_input(void)
 {
+	size_t count = 1;
 	char *bytes;
-	size_t count;
 
 	/* TRANSLATORS: Shown when the next keystroke will be inserted verbatim. */
 	statusbar(_("Verbatim Input"));
@@ -3022,7 +3024,9 @@ void do_verbatim_input(void)
 		if (ISSET(CONSTANT_SHOW))
 			lastmessage = VACUUM;
 
-		inject(bytes, count);
+		if (count < 999)
+			inject(bytes, count);
+
 		wipe_statusbar();
 	} else
 		/* TRANSLATORS: An invalid verbatim Unicode code was typed. */
